@@ -1,5 +1,5 @@
 /*******************************************************************************
- ** Copyright © 2011 - 2022 Petros Koutoupis
+ ** Copyright © 2011 - 2023 Petros Koutoupis
  ** All rights reserved.
  **
  ** This program is free software: you can redistribute it and/or modify
@@ -54,7 +54,7 @@
 	} \
 } while (0)
 
-#define VERSION_STR	"8.2.0"
+#define VERSION_STR	"9.0.0"
 #define DM_MSG_PREFIX	"rapiddisk-cache"
 
 #define READCACHE	1
@@ -175,8 +175,12 @@ int dm_io_async_bvec(unsigned int num_regions, struct dm_io_region *where,
 	struct dm_io_request iorq;
 
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(4,8,0)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(6,0,0)
+	(rw == WRITE) ? (iorq.bi_opf = REQ_OP_WRITE) : (iorq.bi_opf = REQ_OP_READ);
+#else
 	iorq.bi_op = rw;
 	iorq.bi_op_flags = 0;
+#endif
 #else
 	iorq.bi_rw = rw;
 #endif
@@ -1244,7 +1248,7 @@ cache_status(struct dm_target *ti, status_type_t type, unsigned status_flags,
 
 static struct target_type cache_target = {
 	.name    = "rapiddisk-cache",
-	.version = {8, 2, 0},
+	.version = {9, 0, 0},
 	.module  = THIS_MODULE,
 	.ctr	 = cache_ctr,
 	.dtr	 = cache_dtr,
@@ -1286,4 +1290,4 @@ MODULE_LICENSE("GPL");
 MODULE_AUTHOR("Petros Koutoupis <petros@petroskoutoupis.com>");
 MODULE_DESCRIPTION("RapidDisk-Cache DM target is a write-through caching target with RapidDisk volumes.");
 MODULE_VERSION(VERSION_STR);
-MODULE_INFO(Copyright, "Copyright 2010 - 2022 Petros Koutoupis");
+MODULE_INFO(Copyright, "Copyright 2010 - 2023 Petros Koutoupis");
