@@ -1,5 +1,5 @@
 /*******************************************************************************
- ** Copyright © 2011 - 2022 Petros Koutoupis
+ ** Copyright © 2011 - 2023 Petros Koutoupis
  ** All rights reserved.
  **
  ** This program is free software: you can redistribute it and/or modify
@@ -41,13 +41,14 @@
 #include <linux/radix-tree.h>
 #include <linux/io.h>
 
-#define VERSION_STR		"8.2.0"
+#define VERSION_STR		"9.0.0"
 #define PREFIX			"rapiddisk"
 #define BYTES_PER_SECTOR	512
-#define MAX_RDSKS		128
+#define MAX_RDSKS		1024
 #define DEFAULT_MAX_SECTS	127
 #define DEFAULT_REQUESTS	128
 #define GENERIC_ERROR		-1
+#define SUCCESS			0
 
 #define FREE_BATCH		16
 #if LINUX_VERSION_CODE < KERNEL_VERSION(5,15,0)
@@ -57,12 +58,15 @@
 #endif
 
 /* ioctls */
-#define INVALID_CDQUERY_IOCTL	0x5331
-#define INVALID_CDQUERY_IOCTL2	0x5395
-#define RD_GET_STATS		0x0529
+#define IOCTL_INVALID_CDQUERY	0x5331
+#define IOCTL_INVALID_CDQUERY2	0x5395
+#define IOCTL_RD_GET_STATS	0x0529
+#define IOCTL_RD_GET_USAGE	0x0530
+#define IOCTL_RD_BLKFLSBUF	0x0531
 
 static DEFINE_MUTEX(sysfs_mutex);
 static DEFINE_MUTEX(ioctl_mutex);
+static DEFINE_MUTEX(resize_mutex);
 
 struct rdsk_device {
 	int num;
@@ -72,30 +76,29 @@ struct rdsk_device {
 	struct gendisk *rdsk_disk;
 	struct list_head rdsk_list;
 	unsigned long long max_blk_alloc;	/* rdsk: to keep track of highest sector write	*/
+	unsigned long long max_page_cnt;
 	unsigned long long size;
 	unsigned long error_cnt;
 	spinlock_t rdsk_lock;
 	struct radix_tree_root rdsk_pages;
 };
 
-static int rd_ma_no, rd_total; /* no. of attached devices */
-static int rd_max_nr = MAX_RDSKS;
+static unsigned long rd_max_nr = MAX_RDSKS, rd_ma_no, rd_total; /* no. of attached devices */
+static unsigned long rd_size = 0, rd_nr = 0;
 static int max_sectors = DEFAULT_MAX_SECTS, nr_requests = DEFAULT_REQUESTS;
 static LIST_HEAD(rdsk_devices);
 static struct kobject *rdsk_kobj;
-static int rd_nr = 0;
-int rd_size = 0;
 
 module_param(max_sectors, int, S_IRUGO);
 MODULE_PARM_DESC(max_sectors, " Maximum sectors (in KB) for the request queue. (Default = 127)");
 module_param(nr_requests, int, S_IRUGO);
 MODULE_PARM_DESC(nr_requests, " Number of requests at a given time for the request queue. (Default = 128)");
-module_param(rd_nr, int, S_IRUGO);
+module_param(rd_nr, ulong, S_IRUGO);
 MODULE_PARM_DESC(rd_nr, " Maximum number of RapidDisk devices to load on insertion. (Default = 0)");
-module_param(rd_size, int, S_IRUGO);
-MODULE_PARM_DESC(rd_size, " Size of each RAM disk (in KB) loaded on insertion. (Default = 0)");
-module_param(rd_max_nr, int, S_IRUGO);
-MODULE_PARM_DESC(rd_max_nr, " Maximum number of RAM Disks. (Default = 128)");
+module_param(rd_size, ulong, S_IRUGO);
+MODULE_PARM_DESC(rd_size, " Size of each RAM disk (in MB) loaded on insertion. (Default = 0)");
+module_param(rd_max_nr, ulong, S_IRUGO);
+MODULE_PARM_DESC(rd_max_nr, " Maximum number of RAM Disks. (Default = 1024)");
 
 static int rdsk_do_bvec(struct rdsk_device *, struct page *,
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(4,8,0)
@@ -122,33 +125,49 @@ static void rdsk_make_request(struct request_queue *, struct bio *);
 #else
 static int rdsk_make_request(struct request_queue *, struct bio *);
 #endif
-static int attach_device(int);    /* disk size */
-static int detach_device(int);	  /* disk num */
-static int resize_device(int, int); /* disk num, disk size */
+static int attach_device(unsigned long, unsigned long long); /* disk size */
+static int detach_device(unsigned long);                     /* disk num */
+static int resize_device(unsigned long, unsigned long long); /* disk num, disk size */
 static ssize_t mgmt_show(struct kobject *, struct kobj_attribute *, char *);
 static ssize_t mgmt_store(struct kobject *, struct kobj_attribute *,
 			  const char *, size_t);
+static ssize_t devices_show(struct kobject *, struct kobj_attribute *, char *);
 
-static ssize_t mgmt_show(struct kobject *kobj, struct kobj_attribute *attr,
-			 char *buf)
+static ssize_t mgmt_show(struct kobject *kobj, struct kobj_attribute *attr, char *buf)
 {
-	int len;
-	struct rdsk_device *rdsk;
+	int len = 0;
+
+	len = sprintf(buf, "RapidDisk %s\n\nMaximum Number of Attachable Devices: %lu\n",
+		      VERSION_STR, rd_max_nr);
+	len += sprintf(buf + len, "Number of Attached Devices: %lu\nMax Sectors (KB): %d\nNumber of Requests: %d\n",
+		       rd_total, max_sectors, nr_requests);
 
-	len = sprintf(buf, "RapidDisk %s\n\nMaximum Number of Attachable Devices: %d\nNumber of Attached Devices: %d\nMax Sectors (KB): %d\nNumber of Requests: %d\n\n",
-		      VERSION_STR, rd_max_nr, rd_total, max_sectors, nr_requests);
-	list_for_each_entry(rdsk, &rdsk_devices, rdsk_list) {
-		len += sprintf(buf + len, "rd%d\tSize: %llu MBs\tErrors: %lu\n",
-			       rdsk->num, (rdsk->size / 1024 / 1024),
-			       rdsk->error_cnt);
-	}
 	return len;
 }
 
+static ssize_t devices_show(struct kobject *kobj, struct kobj_attribute *attr, char *buf)
+{
+        int len = 0;
+        struct rdsk_device *rdsk;
+
+        mutex_lock(&sysfs_mutex);
+
+        len += sprintf(buf + len, "Device\tSize\tErrors\tUsed\n");
+        list_for_each_entry(rdsk, &rdsk_devices, rdsk_list) {
+                len += scnprintf(buf + len, PAGE_SIZE - len, "rd%d\t%llu\t%lu\t%llu\n", rdsk->num,
+                                 rdsk->size, rdsk->error_cnt, (rdsk->max_page_cnt * PAGE_SIZE));
+        }
+
+        mutex_unlock(&sysfs_mutex);
+        return len;
+}
+
 static ssize_t mgmt_store(struct kobject *kobj, struct kobj_attribute *attr,
 			  const char *buffer, size_t count)
 {
-	int num, size, err = (int)count;
+	int err = (int)count;
+	unsigned long num;
+	unsigned long long size = 0;
 	char *ptr, *buf;
 
 	if (!buffer || count > PAGE_SIZE)
@@ -164,9 +183,10 @@ static ssize_t mgmt_store(struct kobject *kobj, struct kobj_attribute *attr,
 
 	if (!strncmp("rapiddisk attach ", buffer, 17)) {
 		ptr = buf + 17;
-		size = (simple_strtoul(ptr, &ptr, 0) * 2);
+		num = simple_strtoul(ptr, &ptr, 0);
+		size = (simple_strtoull(ptr + 1, &ptr, 0));
 
-		if (attach_device(size) != 0) {
+		if (attach_device(num, size) != SUCCESS) {
 			pr_err("%s: Unable to attach a new RapidDisk device.\n", PREFIX);
 			err = -EINVAL;
 		}
@@ -174,17 +194,17 @@ static ssize_t mgmt_store(struct kobject *kobj, struct kobj_attribute *attr,
 		ptr = buf + 17;
 		num = simple_strtoul(ptr, &ptr, 0);
 
-		if (detach_device(num) != 0) {
-			pr_err("%s: Unable to detach rd%d\n", PREFIX, num);
+		if (detach_device(num) != SUCCESS) {
+			pr_err("%s: Unable to detach rd%lu\n", PREFIX, num);
 			err = -EINVAL;
 		}
 	} else if (!strncmp("rapiddisk resize ", buffer, 17)) {
 		ptr = buf + 17;
 		num = simple_strtoul(ptr, &ptr, 0);
-		size = (simple_strtoul(ptr + 1, &ptr, 0) * 2);
+		size = (simple_strtoull(ptr + 1, &ptr, 0));
 
-		if (resize_device(num, size) != 0) {
-			pr_err("%s: Unable to resize rd%d\n", PREFIX, num);
+		if (resize_device(num, size) != SUCCESS) {
+			pr_err("%s: Unable to resize rd%lu\n", PREFIX, num);
 			err = -EINVAL;
 		}
 	} else {
@@ -201,8 +221,12 @@ write_sysfs_error:
 static struct kobj_attribute mgmt_attribute =
 	__ATTR(mgmt, 0664, mgmt_show, mgmt_store);
 
+static struct kobj_attribute dev_attribute =
+	__ATTR(devices, 0664, devices_show, NULL);
+
 static struct attribute *attrs[] = {
 	&mgmt_attribute.attr,
+	&dev_attribute.attr,
 	NULL,
 };
 
@@ -270,6 +294,7 @@ static struct page *rdsk_insert_page(struct rdsk_device *rdsk, sector_t sector)
 	spin_unlock(&rdsk->rdsk_lock);
 
 	radix_tree_preload_end();
+	rdsk->max_page_cnt++;
 
 	return page;
 }
@@ -280,8 +305,10 @@ static void rdsk_zero_page(struct rdsk_device *rdsk, sector_t sector)
 	struct page *page;
 
 	page = rdsk_lookup_page(rdsk, sector);
-	if (page)
+	if (page) {
 		clear_highpage(page);
+		rdsk->max_page_cnt--;
+	}
 }
 #endif
 
@@ -325,7 +352,7 @@ static int copy_to_rdsk_setup(struct rdsk_device *rdsk,
 		if (!rdsk_insert_page(rdsk, sector))
 			return -ENOSPC;
 	}
-	return 0;
+	return SUCCESS;
 }
 
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,36)
@@ -352,6 +379,7 @@ static void copy_to_rdsk(struct rdsk_device *rdsk, const void *src,
 	page = rdsk_lookup_page(rdsk, sector);
 	BUG_ON(!page);
 
+	mutex_lock(&resize_mutex);
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(3,4,0)
 	dst = kmap_atomic(page);
 #else
@@ -385,6 +413,7 @@ static void copy_to_rdsk(struct rdsk_device *rdsk, const void *src,
 
 	if ((sector + (n / BYTES_PER_SECTOR)) > rdsk->max_blk_alloc)
 		rdsk->max_blk_alloc = (sector + (n / BYTES_PER_SECTOR));
+	mutex_unlock(&resize_mutex);
 }
 
 static void copy_from_rdsk(void *dst, struct rdsk_device *rdsk,
@@ -445,7 +474,7 @@ static int rdsk_do_bvec(struct rdsk_device *rdsk, struct page *page,
 #endif
 			sector_t sector){
 	void *mem;
-	int err = 0;
+	int err = SUCCESS;
 
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(4,8,0)
 	if (is_write) {
@@ -535,7 +564,7 @@ rdsk_make_request(struct request_queue *q, struct bio *bio)
 		goto io_error;
 #endif
 
-	err = 0;
+	err = SUCCESS;
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,336)
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(4,8,0)
 	if (unlikely(bio_op(bio) == REQ_OP_DISCARD)) {
@@ -629,29 +658,30 @@ io_error:
 #endif
 #endif
 #if LINUX_VERSION_CODE < KERNEL_VERSION(3,2,0)
-	return 0;
+	return SUCCESS;
 #endif
 }
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(5,19,0)
+static inline int bdev_openers(struct block_device *bdev)
+{
+	return atomic_read(&bdev->bd_openers);
+}
+#else
+static inline int bdev_openers(struct block_device *bdev)
+{
+	return bdev->bd_openers;
+}
+#endif
+
 static int rdsk_ioctl(struct block_device *bdev, fmode_t mode,
 		      unsigned int cmd, unsigned long arg)
 {
-	loff_t size;
 	int error = 0;
 	struct rdsk_device *rdsk = bdev->bd_disk->private_data;
 
 	switch (cmd) {
-	case BLKGETSIZE:
-		size = bdev->bd_inode->i_size;
-		if ((size >> 9) > ~0UL)
-			return -EFBIG;
-		return copy_to_user((void __user *)arg, &size,
-				    sizeof(size)) ? -EFAULT : 0;
-	case BLKGETSIZE64:
-		return copy_to_user((void __user *)arg,
-				    &bdev->bd_inode->i_size,
-				    sizeof(bdev->bd_inode->i_size)) ? -EFAULT : 0;
-	case BLKFLSBUF:
+	case IOCTL_RD_BLKFLSBUF:
 		/* We are killing the RAM disk data. */
 		mutex_lock(&ioctl_mutex);
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(5,14,0)
@@ -660,7 +690,7 @@ static int rdsk_ioctl(struct block_device *bdev, fmode_t mode,
 		mutex_lock(&bdev->bd_mutex);
 #endif
 		error = -EBUSY;
-		if (bdev->bd_openers <= 1) {
+		if (bdev_openers(bdev) <= 1) {
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(3,3,0)
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(5,8,0) || (defined(RHEL_MAJOR) && RHEL_MAJOR == 8 && RHEL_MINOR >= 4)
 			invalidate_bdev(bdev);
@@ -679,21 +709,21 @@ static int rdsk_ioctl(struct block_device *bdev, fmode_t mode,
 #else
 		mutex_unlock(&bdev->bd_mutex);
 #endif
+		rdsk->max_blk_alloc = 0;
+		rdsk->max_page_cnt = 0;
 		mutex_unlock(&ioctl_mutex);
 		return error;
-	case INVALID_CDQUERY_IOCTL:
-	case INVALID_CDQUERY_IOCTL2:
+	case IOCTL_INVALID_CDQUERY:
+	case IOCTL_INVALID_CDQUERY2:
 		return -EINVAL;
-	case RD_GET_STATS:
+	case IOCTL_RD_GET_STATS:
 		return copy_to_user((void __user *)arg,
 			&rdsk->max_blk_alloc,
 			sizeof(rdsk->max_blk_alloc)) ? -EFAULT : 0;
-	case BLKPBSZGET:
-	case BLKBSZGET:
-	case BLKSSZGET:
-		size = BYTES_PER_SECTOR;
-		return copy_to_user((void __user *)arg, &size,
-			sizeof(size)) ? -EFAULT : 0;
+	case IOCTL_RD_GET_USAGE:
+		return copy_to_user((void __user *)arg,
+			&rdsk->max_page_cnt,
+			sizeof(rdsk->max_page_cnt)) ? -EFAULT : 0;
 	}
 
 	pr_warn("%s: 0x%x invalid ioctl.\n", PREFIX, cmd);
@@ -708,38 +738,36 @@ static const struct block_device_operations rdsk_fops = {
 	.ioctl = rdsk_ioctl,
 };
 
-static int attach_device(int size)
+static int attach_device(unsigned long num, unsigned long long size)
 {
-	int num = 0;
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(5,16,0)
 	int err = GENERIC_ERROR;
 #endif
 	struct rdsk_device *rdsk, *tmp;
 	struct gendisk *disk;
-	unsigned char *string, name[16];
+	sector_t sectors = 0;
+
+	if (num > MINORMASK) {
+		pr_warn("%s: Reached maximum value for the attached disk.\n", PREFIX);
+		goto out;
+	}
 
 	if (rd_total >= rd_max_nr) {
-		pr_warn("%s: Reached maximum number of attached disks.\n",
-			PREFIX);
+		pr_warn("%s: Reached maximum number of attached disks.\n", PREFIX);
 		goto out;
 	}
 
-	string = kzalloc(PAGE_SIZE, GFP_KERNEL);
-	if (!string)
+	if (size % BYTES_PER_SECTOR != 0) {
+		pr_err("%s: Invalid size input. Size must be a multiple of sector size %d.\n",
+		       PREFIX, BYTES_PER_SECTOR);
 		goto out;
-	list_for_each_entry(tmp, &rdsk_devices, rdsk_list) {
-		sprintf(string, "%srd%d,", string, tmp->num);
-		num++;
 	}
-	while (num >= 0) {
-		memset(name, 0x0, sizeof(name));
-		sprintf(name, "rd%d,", num);
-                if (strstr(string, (const char *)name) == NULL) {
-                        break;
-                }
-                num--;
-        }
-	kfree(string);
+	sectors = (size / BYTES_PER_SECTOR);
+
+
+	list_for_each_entry(tmp, &rdsk_devices, rdsk_list) {
+		if (tmp->num == num) goto out;
+	}
 
 	rdsk = kzalloc(sizeof(*rdsk), GFP_KERNEL);
 	if (!rdsk)
@@ -747,7 +775,8 @@ static int attach_device(int size)
 	rdsk->num = num;
 	rdsk->error_cnt = 0;
 	rdsk->max_blk_alloc = 0;
-	rdsk->size = ((unsigned long long)size * BYTES_PER_SECTOR);
+	rdsk->max_page_cnt = 0;
+	rdsk->size = size;
 	spin_lock_init(&rdsk->rdsk_lock);
 	INIT_RADIX_TREE(&rdsk->rdsk_pages, GFP_ATOMIC);
 
@@ -805,7 +834,11 @@ static int attach_device(int size)
 	disk->queue->nr_requests = nr_requests;
 	disk->queue->limits.discard_granularity = PAGE_SIZE;
 	disk->queue->limits.max_discard_sectors = UINT_MAX;
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(5,19,0)
+	blk_queue_max_discard_sectors(disk->queue, 0);
+#else
 	blk_queue_flag_set(QUEUE_FLAG_DISCARD, disk->queue);
+#endif
 	blk_queue_flag_set(QUEUE_FLAG_NONROT, disk->queue);
 #else
 	rdsk->rdsk_queue->limits.max_sectors = (max_sectors * 2);
@@ -844,8 +877,8 @@ static int attach_device(int size)
 #else
 	disk->flags |= GENHD_FL_SUPPRESS_PARTITION_INFO;
 #endif
-	sprintf(disk->disk_name, "rd%d", num);
-	set_capacity(disk, size);
+	sprintf(disk->disk_name, "rd%lu", num);
+	set_capacity(disk, sectors);
 
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(5,16,0)
 	err = add_disk(disk);
@@ -856,9 +889,8 @@ static int attach_device(int size)
 #endif
 	list_add_tail(&rdsk->rdsk_list, &rdsk_devices);
 	rd_total++;
-	pr_info("%s: Attached rd%d of %llu bytes in size.\n", PREFIX,
-		num, rdsk->size);
-	return 0;
+	pr_info("%s: Attached rd%lu of %llu bytes in size.\n", PREFIX, num, rdsk->size);
+	return SUCCESS;
 
 out_free_queue:
 #if LINUX_VERSION_CODE < KERNEL_VERSION(5,14,0)
@@ -870,7 +902,7 @@ out:
 	return GENERIC_ERROR;
 }
 
-static int detach_device(int num)
+static int detach_device(unsigned long num)
 {
 	struct rdsk_device *rdsk;
 	bool found = false;
@@ -893,35 +925,47 @@ static int detach_device(int num)
 	rdsk_free_pages(rdsk);
 	kfree(rdsk);
 	rd_total--;
-	pr_info("%s: Detached rd%d.\n", PREFIX, num);
+	pr_info("%s: Detached rd%lu.\n", PREFIX, num);
 
-	return 0;
+	return SUCCESS;
 }
 
-static int resize_device(int num, int size)
+static int resize_device(unsigned long num, unsigned long long size)
 {
 	struct rdsk_device *rdsk;
 	bool found = false;
+	sector_t sectors = 0;
 
+	if (size % BYTES_PER_SECTOR != 0) {
+		pr_err("%s: Invalid size input. Size must be a multiple of sector size %d.\n",
+		       PREFIX, BYTES_PER_SECTOR);
+		return GENERIC_ERROR;
+	}
+	sectors = (size / BYTES_PER_SECTOR);
+
+	mutex_lock(&resize_mutex);
 	list_for_each_entry(rdsk, &rdsk_devices, rdsk_list)
 		if (rdsk->num == num) {
 			found = true;
 			break;
 		}
 
-	if (!found)
-		return GENERIC_ERROR;
-
-	if (size <= get_capacity(rdsk->rdsk_disk)) {
-		pr_warn("%s: Please specify a larger size for resizing.\n",
-			PREFIX);
+	if (!found) {
+		mutex_unlock(&resize_mutex);
 		return GENERIC_ERROR;
 	}
-	set_capacity(rdsk->rdsk_disk, size);
-	rdsk->size = (size * BYTES_PER_SECTOR);
-	pr_info("%s: Resized rd%d of %lu bytes in size.\n", PREFIX, num,
-	        (unsigned long)(size * BYTES_PER_SECTOR));
-	return 0;
+
+	if (size <= (rdsk->max_blk_alloc * BYTES_PER_SECTOR)) {
+		pr_warn("%s: Please specify a larger size for resizing.\n",
+			PREFIX);
+		mutex_unlock(&resize_mutex);
+		return GENERIC_ERROR;
+	}
+	set_capacity(rdsk->rdsk_disk, sectors);
+	rdsk->size = size;
+	mutex_unlock(&resize_mutex);
+	pr_info("%s: Resized rd%lu of %llu bytes in size.\n", PREFIX, num, size);
+	return SUCCESS;
 }
 
 static int __init init_rd(void)
@@ -931,7 +975,7 @@ static int __init init_rd(void)
 	rd_total = rd_ma_no = 0;
 	rd_ma_no = register_blkdev(rd_ma_no, PREFIX);
 	if (rd_ma_no < 0) {
-		pr_err("%s: Failed registering rdsk, returned %d\n",
+		pr_err("%s: Failed registering rdsk, returned %lu\n",
 		       PREFIX, rd_ma_no);
 		return rd_ma_no;
 	}
@@ -944,14 +988,14 @@ static int __init init_rd(void)
 		goto init_failure2;
 
 	for (i = 0; i < rd_nr; i++) {
-		retval = attach_device(rd_size * 2);
+		retval = attach_device(i, rd_size * 2048);
 		if (retval) {
 			pr_err("%s: Failed to load RapidDisk volume rd%d.\n",
 			       PREFIX, i);
 			goto init_failure2;
 		}
 	}
-	return 0;
+	return SUCCESS;
 
 init_failure2:
 	kobject_put(rdsk_kobj);
@@ -977,4 +1021,4 @@ MODULE_LICENSE("GPL");
 MODULE_AUTHOR("Petros Koutoupis <petros@petroskoutoupis.com>");
 MODULE_DESCRIPTION("RapidDisk is an enhanced RAM disk block device driver.");
 MODULE_VERSION(VERSION_STR);
-MODULE_INFO(Copyright, "Copyright 2010 - 2022 Petros Koutoupis");
+MODULE_INFO(Copyright, "Copyright 2010 - 2023 Petros Koutoupis");
