@@ -1,6 +1,6 @@
 /* rxflush.c */
 
-/** Copyright © 2016 - 2022 Petros Koutoupis
+/** Copyright © 2016 - 2023 Petros Koutoupis
  ** All rights reserved.
  **
  ** This program is free software: you can redistribute it and/or modify
@@ -30,16 +30,18 @@
 #include <errno.h>
 #include <string.h>
 
-int main (){
+#define IOCTL_RD_BLKFLSBUF	0x0531
+
+int main () {
 	int fd, rc;
 
-	if((fd = open("/dev/rd0", O_WRONLY)) < 0){
+	if ((fd = open("/dev/rd0", O_WRONLY)) < 0) {
 		printf("%s\n", strerror(errno));
 		return errno;
 	}
 
-	rc = ioctl(fd, BLKFLSBUF, 0);
-	printf("Sent ioctl() BLKFLSBUF and got return: %d\n", rc);
+	rc = ioctl(fd, IOCTL_RD_BLKFLSBUF, 0);
+	printf("Sent ioctl() IOCTL_RD_BLKFLSBUF and got return: %d\n", rc);
 
 	close (fd);
 
