@@ -1,6 +1,6 @@
 /* rxioctl.c */
 
-/** Copyright © 2016 - 2022 Petros Koutoupis
+/** Copyright © 2016 - 2023 Petros Koutoupis
  ** All rights reserved.
  **
  ** This program is free software: you can redistribute it and/or modify
@@ -31,24 +31,40 @@
 #include <string.h>
 #include <malloc.h>
 
-#define RD_GET_STATS     0x0529
+#define IOCTL_RD_GET_STATS     0x0529
+#define IOCTL_RD_GET_USAGE     0x0530
 
-int main (){
+int main () {
 	int fd, max_sectors;
+	unsigned long long max_usage;
 
-	if((fd = open("/dev/rd0", O_WRONLY)) < 0){
+	if ((fd = open("/dev/rd0", O_WRONLY)) < 0) {
 		printf("%s\n", strerror(errno));
 		return errno;
 	}
 
-	if(ioctl(fd, RD_GET_STATS, &max_sectors) == -1){
+	if (ioctl(fd, IOCTL_RD_GET_STATS, &max_sectors) == -1) {
 		printf("%s\n", strerror(errno));
 		return errno;
-	}else{
+	} else {
 		printf("max sectors allocated: %d\n", max_sectors);
 	}
 
 	close (fd);
 
+	if ((fd = open("/dev/rd0", O_WRONLY)) < 0) {
+		printf("%s\n", strerror(errno));
+		return errno;
+	}
+
+	if (ioctl(fd, IOCTL_RD_GET_USAGE, &max_usage) == -1) {
+		printf("%s\n", strerror(errno));
+		return errno;
+	} else {
+		printf("max pages allocated: %llu\n", max_usage);
+	}
+
+	close (fd);
+
 	return 0;
 }
